

PROGRAM Readump;
{ portions of code are from J. Duntemann's 'Complete Turbo Pascal' }

USES DOS,CRT;


CONST
  COM2INT = 11;       { Vector # for COM2: (IRQ3) }
  COM1INT = 12;       { Vector # for COM1: (IRQ4) }

  { 8250 control registers, masks, etc. (COM2)}
  { ***** change $2F8 etc. to $3F8 etc. for COM1 ***** }
  RBR      = $2F8;    { 8250 Receive Buffer Register     }
  THR      = $2F8;    { 8250 Transmit Holding Register   }
  LCR      = $2FB;    { 8250 Line Control Register       }
  IER      = $2F9;    { 8250 Interrupt Enable Register   }
  MCR      = $2FC;    { 8250 Modem Control Register      }
  LSR      = $2FD;    { 8250 Line Status Register        }
  DLL      = $2F8;    { 8250 Divisor Latch LSB           }
  DLM      = $2F9;    { 8250 Divisor Latch MSB           }
  DLAB     = $80;     { 8250 Divisor Latch Access Bit    }


  BAUD4800 = 24;      { Value for 4800 baud operation    }
  NOPARITY = 0;       { Comm format value for no parity  }
  BITS8    = $03;     { Comm format value for 8 bits     }
  DTR      = $01;     { Value for Data Terminal Ready    }
  RTS      = $02;     { value for Ready To Send          }
  OUT2     = $08;     { Bit that enables adapter interrupts }

  { 8259 control registers, masks, etc. }
  OCW1     = $21;     { 8259 Operation Control Word 1    }
  OCW2     = $20;     { 8259 Operation Control Word 2    }
  IRQ4     = $10;     { Mask to turn IRQ4 interrupts on/off }
  IRQ3     = $08;     { Mask to turn IRQ3 interrupts on/off }



TYPE
  CircularBuffer = ARRAY[0..1023] OF Char;  { A 1K input buffer }


VAR
  ch         : Char;              { Character from keyboard }
  CommChar   : Char;              { Character from the comm port }
  Clearit    : Byte;              { Dummy variable }
  Buffer     : CircularBuffer;    { Our incoming character buffer }
  LastRead,                       { Index of the last character read }
  LastSaved  : Integer;           { Index of the last character stored }
  OldVector  : Pointer;           { Global storage slot for the old }
                                  { interrupt vector }
  totbytes   : Integer;           { total bytes to input }
  fname      : String[20];        { filename }
  datafile   : Text;              { file that holds data }
  j,k        : Integer;           { counters }
  DegF       : Boolean;           { write temperature in degrees F? }
  onet       : Real;              { one temperature measurement }
  terr       : Integer;           { temperature error }
  value      : Byte;              { value Byte }
  sign       : Byte;              { sign Byte }
  remain     : Byte;              { remain Byte }
  slope      : Byte;              { slope Byte }
  inarray    : ARRAY[0..8191] OF Byte;   { holds incomming data }

PROCEDURE EnableInterrupts;

INLINE($FB);




PROCEDURE Incoming(Flags,CS,IP,AX,BX,CX,DX,SI,DI,DS,ES,BP : Word);
INTERRUPT;

BEGIN
  EnableInterrupts;
  IF LastSaved >= 1023 THEN LastSaved := 0 ELSE Inc(LastSaved);
  Buffer[LastSaved] := Char(Port[RBR]);
  Port[OCW2] := $20;                    { Send EOI byte to 8259 }
END;



{$F+}
PROCEDURE JTermExitProc;

BEGIN
  Port[IER] := 0;                      { Disable interrupts at 8250 }
  Port[OCW1] := Port[OCW1] OR IRQ3;          { Disable IRQ4 at 8259 }
     { ***** change to OR IRQ4 for COM1 ***** }
  Port[MCR] := 0;                       { Bring the comm line down  }
  SetIntVec(Com2Int,OldVector);   { Restore previously saved vector }
     { ***** change to Com1Int for COM1 ***** } 
END;
{$F-}



PROCEDURE SetupSerialPort;

BEGIN
  LastRead  := 0;  { Initialize the circular buffer pointers }
  LastSaved := 0;

  Port[IER] := 0;  { Disable interrupts while we're setting them up }

  GetIntVec(Com2Int,OldVector);              { Save old IRQ3 vector }
     { ***** change to Com1Int for COM1 ***** }
  ExitProc := @JTermExitProc;           { Hook exit proc into chain }
  SetIntVec(Com2Int,@Incoming); { Put ISR address into vector table }

  Port[LCR] := Port[LCR] OR DLAB;  { Set up 8250 to set baud rate   }
  Port[DLL] := Lo(Baud4800);        { Set baud rate divisor          }
  Port[DLM] := Hi(Baud4800);
  Port[LCR] := BITS8 OR NOPARITY;      { Set word length and parity }
  Port[MCR] := DTR OR RTS OR OUT2;     { Enable adapter, DTR, & RTS }
  Port[OCW1] := Port[OCW1] AND (NOT IRQ3); { Turn on 8259 IRQ3 ints }
      { ***** change to NOT IRQ4 for COM1 ***** }
  Clearit := Port[RBR];                { Clear any garbage from RBR }
  Clearit := Port[LSR];                { Clear any garbage from LSR }

  Port[IER] := $01;        { Enable interrupt on received character }
END;


FUNCTION InStat : Boolean;

BEGIN
  IF LastSaved <> LastRead THEN InStat := True
    ELSE InStat := False;
END;


FUNCTION InChar : Char;    { Bring in the next character }
                           {   from the ring buffer }
BEGIN
  IF LastRead >= 1023 THEN LastRead := 0
    ELSE LastRead := Succ(LastRead);
  InChar := Buffer[LastRead];
END;

BEGIN

  ClrScr;
  Writeln('***** Read data saved in temperature datalogger *****');
  Writeln('   ***** connect datalogger to PC COM2 port *****');
  Writeln;
  Write('Input Filename: ');
  Readln(fname);
  Assign(datafile,fname);
  {$i-} ReWrite(datafile); {$i+}
  IF IOResult <> 0 THEN BEGIN
    Writeln('Can''t open '+fname);
    Halt;
  END;
  Write('Change to degrees F? (y/n)');
  Readln(ch);
  IF UpCase(ch) = 'Y' THEN DegF := TRUE ELSE DegF := FALSE; 
  Writeln;
  Writeln('***** Press button on datalogger to start dump *****');
  SetupSerialPort;              { Set up serial port & turn on interrupts }
  WHILE NOT InStat DO;      { Wait for a character to come in }
  totbytes := Ord(Inchar);
  WHILE NOT InStat DO;      { Wait for a character to come in }
  totbytes := totbytes + Ord(Inchar) * 256;
  IF totbytes = 0 THEN BEGIN Writeln('No Data'); Halt; END;
  j := 0;
  REPEAT
     WHILE NOT InStat DO;                { Wait for a character to come in }
     inarray[j] := Ord(InChar);                        { Go get character  }
     Inc(j);  
  UNTIL j >= totbytes;
  j := 0;
  REPEAT
     value := inarray[j];
     sign  := inarray[j+1];
     remain := inarray[j+2];
     slope := inarray[j+3];
     IF sign = $FF THEN
        BEGIN
           value := (value SHR 1) + 128;
           onet := -(Succ(value XOR $FF) - 0.75 + (remain/slope));
        END 
        ELSE
          onet := (value SHR 1) +0.75 -(remain/slope); 
     IF DegF THEN onet := (9 * onet) / 5 + 32;
     Write(j DIV 4:5,' -',onet:6:1);
     Writeln(datafile,onet:6:1);
     IF DegF THEN writeln(' Deg F') ELSE writeln(' Deg C');
     Inc(j,4); 
  UNTIL j >= totbytes; 
  Close(datafile);
END.
